Function std::hash::is_hash_trivial
pub fn is_hash_trivial<T>() -> bool
where
T: Hash,
Returns true if the type T is trivially hashable.
See [Hash::is_hash_trivial] for the exact meaning and the caveats.
pub fn is_hash_trivial<T>() -> bool
where
T: Hash,
Returns true if the type T is trivially hashable.
See [Hash::is_hash_trivial] for the exact meaning and the caveats.